@@ -76,6 +76,8 @@ private:
 	UInt32						fModeIndex;
 	bool						fTheatreMode;
 	bool						fAreChangingMode;
+	bool						fIsSystemSleeping;
+	bool						fIsDisplayOff;
 
 public:
 	virtual bool		start(	IOService* provider );
@@ -119,6 +121,8 @@ private:
 
 	bool						fTheatreMode;
 	bool						fAreChangingMode;
+	bool						fIsSystemSleeping;
+	bool						fIsDisplayOff;
 
 public:
 	virtual	bool		start( IOService* provider );
@@ -148,27 +152,27 @@ private:
 
 
 class AppleVCP : public IODisplayParameterHandler
-{
-    OSDeclareDefaultStructors(AppleVCP);
+	{
+	OSDeclareDefaultStructors( AppleVCP );
 
 private:
-    OSDictionary *	fDisplayParams;
-    UInt32 		fIICAddress;
+	OSDictionary*			fDisplayParams;
+	UInt32					fIICAddress;
 
 public:
-    virtual bool start(	IOService * provider );
+	virtual bool start(	IOService* provider );
 
-    virtual bool setDisplay( IODisplay * display );
-    virtual bool doIntegerSet( OSDictionary * params,
-                               const OSSymbol * paramName, UInt32 value );
-    virtual bool doDataSet( const OSSymbol * paramName, OSData * value );
-    virtual bool doUpdate( void );
+	virtual bool setDisplay( IODisplay* display );
+	virtual bool doIntegerSet( OSDictionary* params, const OSSymbol* paramName, UInt32 value );
+	virtual bool doDataSet( const OSSymbol* paramName, OSData* value );
+	virtual bool doUpdate( void );
 
 private:
-    bool readVCPRange( UInt8 command, SInt32 * max, SInt32 * current );
-    bool setupVCP( void );
+	bool getSymbolRange( const OSSymbol* sym, SInt32* oMin, SInt32* oMax, SInt32* oCurrent );
+	bool readVCPRange( UInt8 command, SInt32* max, SInt32* current );
+	bool setupVCP( void );
+	};
 
-};
 
 class AppleG3AIO : public IODisplayParameterHandler
 {
@@ -463,6 +467,8 @@ bool AppleIVAD::start( IOService* provider )
 	fIVADPrefsKey = OSSymbol::withCString( "geometry" );
 
 	fAreChangingMode = false;
+	fIsSystemSleeping = false;
+	fIsDisplayOff = false;
 
 	// Install handler for shutdown notifications
 
@@ -799,14 +805,13 @@ IOReturn AppleIVAD::_powerDownHandler( void* target, void* refCon, UInt32 messag
 IOReturn AppleIVAD::powerDownHandler( void* refCon, UInt32 messageType, IOService* provider, void* messageArgument, vm_size_t argSize )
 	{
 	IOReturn				status = kIOReturnUnsupported;
+	UInt8					regValue;
 
 	switch ( messageType )
 		{
 		case	kIOMessageSystemWillRestart:
 		case	kIOMessageSystemWillPowerOff:
 			{
-			UInt8					regValue;
-
 			// We need to set the contrast to 0 on restarts and shutdown, to minimize the flash
 			// that occurs when the power supply is turning off.
 
@@ -818,10 +823,27 @@ IOReturn AppleIVAD::powerDownHandler( void* refCon, UInt32 messageType, IOServic
 			}
 
 		case	kIOMessageSystemWillSleep:
-		default:
 			{
-			// We don't do anything at sleep time. That is handled via standard power management
-			// functions. See frameBufferEvent() for more information.
+			fIsSystemSleeping = true;
+			status = kIOReturnSuccess;
+
+			if ( fIsDisplayOff )
+				{
+				// If the display is already off, then we won't get another frame buffer notification
+				// telling us to turn off.  So, we need to turn off the PLL1 register here.
+
+				regValue = 0;
+				ApplePMUWriteIIC( kPowerSupplyBus, fIICAddress, kIVADPLL1, &regValue, sizeof( regValue ) );
+				}
+
+			break;
+			}
+
+		case	kIOMessageSystemHasPoweredOn:
+			{
+			fIsSystemSleeping = false;
+			status = kIOReturnSuccess;
+
 			break;
 			}
 		}
@@ -886,20 +908,38 @@ IOReturn AppleIVAD::framebufferEvent( IOFramebuffer* framebuffer, IOIndex event,
 
 		case	kIOFBNotifyWillPowerOn:
 			{
-			regValue = fEEPROM->stdParams.pll1FVC;
-			ApplePMUWriteIIC( kPowerSupplyBus, fIICAddress, kIVADPLL1, &regValue, sizeof( regValue ) );
+			if ( fIsSystemSleeping )
+				{
+				//
+				// #2983513
+				// Restore the PLL1 register.
+				//
+
+				regValue = fEEPROM->stdParams.pll1FVC;
+				ApplePMUWriteIIC( kPowerSupplyBus, fIICAddress, kIVADPLL1, &regValue, sizeof( regValue ) );
+				fIsSystemSleeping = false;
+				}
+
+			fIsDisplayOff = false;
 			break;
 			}
 
 		case	kIOFBNotifyDidPowerOff:
 			{
-			regValue = 0;
-            ApplePMUWriteIIC( kPowerSupplyBus, fIICAddress, kIVADPLL1, &regValue, sizeof( regValue ) );
+			if ( fIsSystemSleeping )
+				{
+				//
+				// #2983513
+				// Put PLL1 register in free-running mode.  Only do this at system sleep time.
+				//
+
+				regValue = 0;
+				ApplePMUWriteIIC( kPowerSupplyBus, fIICAddress, kIVADPLL1, &regValue, sizeof( regValue ) );
+				}
+
+			fIsDisplayOff = true;
 			break;
 			}
-
-		default:
-			break;
 		}
 
 	return( kIOReturnSuccess );
@@ -907,7 +947,7 @@ IOReturn AppleIVAD::framebufferEvent( IOFramebuffer* framebuffer, IOIndex event,
 
 
 bool AppleIVAD::doIntegerSet( OSDictionary * params, const OSSymbol * sym, UInt32 value )
-{
+	{
 	OSNumber*					num;
 	UInt32						reg;
 	UInt8						regValue;
@@ -922,7 +962,7 @@ bool AppleIVAD::doIntegerSet( OSDictionary * params, const OSSymbol * sym, UInt3
 		fUserPrefs->brightness		= fEEPROM->stdParams.brightness;
 		fUserPrefs->parallelogram	= fEEPROM->stdParams.parallelogram;
 		fUserPrefs->tilt			= fEEPROM->stdParams.tilt;
-        return( true );
+		return( true );
 		}
 	else
 		if ( sym == gIODisplayParametersCommitKey )
@@ -996,11 +1036,11 @@ bool AppleIVAD::doIntegerSet( OSDictionary * params, const OSSymbol * sym, UInt3
 											fUserPrefs->parallelogram = regValue;
 										else
 											if ( sym == gIODisplayRotationKey )
-            fUserPrefs->tilt = regValue;
-    }
+												fUserPrefs->tilt = regValue;
+		}
 
-    return( ok );
-}
+	return( ok );
+	}
 
 
 bool AppleIVAD2::start( IOService* provider )
@@ -1067,6 +1107,8 @@ bool AppleIVAD2::start( IOService* provider )
 		}
 
 	fAreChangingMode = false;
+	fIsSystemSleeping = false;
+	fIsDisplayOff = false;
 
 	// Install handler for shutdown notifications
 
@@ -1300,7 +1342,7 @@ IOReturn AppleIVAD2::doI2CReadWrite( const char* functionName, UInt8 bus, UInt8
 		return( kIOReturnError );
 		}
 
-	status = pmuI2CDriver->callPlatformFunction( "openI2CBus", true, ( void * ) bus, NULL, NULL, NULL );
+	status = pmuI2CDriver->callPlatformFunction( "openI2CBus", true, ( void * ) ( ( UInt32 ) bus ), NULL, NULL, NULL );
 
 	if ( status != kIOReturnSuccess )
 		{
@@ -1326,7 +1368,7 @@ IOReturn AppleIVAD2::doI2CReadWrite( const char* functionName, UInt8 bus, UInt8
 
 	adjustedAddress = address >> 1;
 
-	status = pmuI2CDriver->callPlatformFunction( functionName, true, ( void * ) adjustedAddress, ( void * ) subAddress, ( void * ) buffer, ( void * ) count );
+	status = pmuI2CDriver->callPlatformFunction( functionName, true, ( void * ) ( ( UInt32 ) adjustedAddress ), ( void * ) ( ( UInt32 ) subAddress ), ( void * ) buffer, ( void * ) count );
 
 	( void ) pmuI2CDriver->callPlatformFunction( "closeI2CBus", true, NULL, NULL, NULL, NULL );
 
@@ -1639,14 +1681,13 @@ IOReturn AppleIVAD2::_powerDownHandler( void* target, void* refCon, UInt32 messa
 IOReturn AppleIVAD2::powerDownHandler( void* refCon, UInt32 messageType, IOService* provider, void* messageArgument, vm_size_t argSize )
 	{
 	IOReturn				status = kIOReturnUnsupported;
+	UInt8					regValue;
 
 	switch ( messageType )
 		{
 		case	kIOMessageSystemWillRestart:
 		case	kIOMessageSystemWillPowerOff:
 			{
-			UInt8					regValue;
-
 			// On a restart or shutdown, we need to: set the contrast to 0 and set all the cutoff
 			// PWM registers to 0.  This minimizes the flash that occurs when the power supply is turning off.
 
@@ -1662,10 +1703,27 @@ IOReturn AppleIVAD2::powerDownHandler( void* refCon, UInt32 messageType, IOServi
 			}
 
 		case	kIOMessageSystemWillSleep:
-		default:
 			{
-			// We don't do anything at sleep time. That is handled via standard power management
-			// functions. See frameBufferEvent() for more information.
+			fIsSystemSleeping = true;
+			status = kIOReturnSuccess;
+
+			if ( fIsDisplayOff )
+				{
+				// If the display is already off, then we won't get another frame buffer notification
+				// telling us to turn off.  So, we need to turn off the PLL1 register here.
+
+				regValue = 0;
+				writeI2CData( kPowerSupplyBus, fIICAddress, kIVADPLL1, &regValue, sizeof( regValue ) );
+				}
+
+			break;
+			}
+
+		case	kIOMessageSystemHasPoweredOn:
+			{
+			fIsSystemSleeping = false;
+			status = kIOReturnSuccess;
+
 			break;
 			}
 		}
@@ -1735,8 +1793,17 @@ IOReturn AppleIVAD2::framebufferEvent( IOFramebuffer* framebuffer, IOIndex event
 
 		case	kIOFBNotifyWillPowerOn:
 			{
-			regValue = fEEPROM->stdParams.pll1FVC;
-			writeI2CData( kPowerSupplyBus, fIICAddress, kIVADPLL1, &regValue, sizeof( regValue ) );
+			if ( fIsSystemSleeping )
+				{
+				//
+				// #2983513
+				// Restore the PLL1 register.
+				//
+
+				regValue = fEEPROM->stdParams.pll1FVC;
+				writeI2CData( kPowerSupplyBus, fIICAddress, kIVADPLL1, &regValue, sizeof( regValue ) );
+				fIsSystemSleeping = false;
+				}
 
 			regValue = fEEPROM->stdParams.cutoff1;
 			writeI2CData( kPowerSupplyBus, fPWMIICAddress, kIVADPWMCutoff1, &regValue, sizeof( regValue ) );
@@ -1746,6 +1813,8 @@ IOReturn AppleIVAD2::framebufferEvent( IOFramebuffer* framebuffer, IOIndex event
 
 			regValue = fEEPROM->stdParams.cutoff3;
 			writeI2CData( kPowerSupplyBus, fPWMIICAddress, kIVADPWMCutoff3, &regValue, sizeof( regValue ) );
+
+			fIsDisplayOff = false;
 			break;
 			}
 
@@ -1756,12 +1825,20 @@ IOReturn AppleIVAD2::framebufferEvent( IOFramebuffer* framebuffer, IOIndex event
 			writeI2CData( kPowerSupplyBus, fPWMIICAddress, kIVADPWMCutoff1, &regValue, sizeof( regValue ) );
 			writeI2CData( kPowerSupplyBus, fPWMIICAddress, kIVADPWMCutoff2, &regValue, sizeof( regValue ) );
 			writeI2CData( kPowerSupplyBus, fPWMIICAddress, kIVADPWMCutoff3, &regValue, sizeof( regValue ) );
-			writeI2CData( kPowerSupplyBus, fIICAddress, kIVADPLL1, &regValue, sizeof( regValue ) );
+
+			if ( fIsSystemSleeping )
+				{
+				//
+				// #2983513
+				// Put PLL1 register in free-running mode.  Only do this at system sleep time.
+				//
+
+				writeI2CData( kPowerSupplyBus, fIICAddress, kIVADPLL1, &regValue, sizeof( regValue ) );
+				}
+
+			fIsDisplayOff = true;
 			break;
 			}
-
-		default:
-			break;
 		}
 
 	return( kIOReturnSuccess );
@@ -1875,248 +1952,349 @@ bool AppleIVAD2::doIntegerSet( OSDictionary* params, const OSSymbol* sym, UInt32
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-bool AppleVCP::start( IOService * provider )
-{
-    OSData * data;
+bool AppleVCP::start( IOService* provider )
+	{
+	OSData*							data;
 
-    if( !super::start( provider))
-        return( false );
+	if ( !super::start( provider ) )
+		return( false );
 
-    if( 0 == (data = OSDynamicCast(OSData,
-                    provider->getProperty("iic-address"))))
-        return( false );
+	if ( 0 == (data = OSDynamicCast( OSData, provider->getProperty( "iic-address" ) ) ) )
+		return( false );
 
-    fDisplayParams = OSDynamicCast(OSDictionary, getProperty(gIODisplayParametersKey));
-    if( !fDisplayParams)
-        return( false );
-        
-    fIICAddress = *((UInt32 *) data->getBytesNoCopy());
+	fDisplayParams = OSDynamicCast( OSDictionary, getProperty( gIODisplayParametersKey ) );
+	if ( !fDisplayParams )
+		return( false );
 
-    if( !setupVCP())
-        return( false );
+	fIICAddress = *( ( UInt32 * ) data->getBytesNoCopy() );
 
-    return( true );
-}
+	if ( !setupVCP() )
+		return( false );
 
-bool AppleVCP::setDisplay( IODisplay * display )
-{
-    OSNumber * num;
+	return( true );
+	}
 
-    display->setProperty( gIODisplayParametersKey, fDisplayParams);
 
-    num = OSNumber::withNumber( kAppleOnboardGUID, 64 );
-    display->setProperty( gIODisplayGUIDKey, num);
-    num->release();
+bool AppleVCP::setDisplay( IODisplay* display )
+	{
+	OSNumber*						num;
+
+	display->setProperty( gIODisplayParametersKey, fDisplayParams );
+
+	num = OSNumber::withNumber( kAppleOnboardGUID, 64 );
+	display->setProperty( gIODisplayGUIDKey, num );
+	num->release();
+
+	return( true );
+	}
+
+
+bool AppleVCP::getSymbolRange( const OSSymbol* sym, SInt32* oMin, SInt32* oMax, SInt32* oCurrent )
+	{
+	OSNumber*						num;
+	OSDictionary*					params;
+	UInt32							command;
+	bool							success = false;
+
+	if ( ( params = OSDynamicCast( OSDictionary, fDisplayParams->getObject( sym ) ) ) == NULL )
+		return( false );
+
+	if ( ( num = ( OSNumber * ) params->getObject( "reg" ) ) == NULL )
+		return( false );
+
+	*oMin = 0;
+
+	command = num->unsigned32BitValue();
+	success = readVCPRange( command, oMax, oCurrent );
+
+	return( success );
+	}
 
-    return( true );
-}
 
 bool AppleVCP::doUpdate( void )
-{
-    return( false );
-}
+	{
+	OSIterator*						iter;
 
-bool AppleVCP::doDataSet( const OSSymbol * paramName, OSData * value )
-{
-    return( false );
-}
+	iter = OSCollectionIterator::withCollection( fDisplayParams );
+	if ( iter )
+		{
+		const OSSymbol*						sym;
 
-enum {
-    kGetVCPFeature		= 0x01,
-    kReplyVCPFeature		= 0x02,
-    kSetVCPFeature		= 0x03,
-    kGetControlKeyMode		= 0x04,
-    kReplyControlKeyMode	= 0x05,
-    kSetControlKeyMode		= 0x06,
-    kGetTimingReport		= 0x07,
-    kGetKeyReport		= 0x08,
-    kResetVCPFeature		= 0x09,
-    kDisableVCPFeature		= 0x0A,
-    kEnableVCPFeature		= 0x0B,
-    kSaveCurrentSettings	= 0x0C
-};
+		while( ( sym = ( const OSSymbol * ) iter->getNextObject() ) )
+			{
+			SInt32							current;
+			SInt32							max;
+			SInt32							min;
+
+			if ( sym == gIODisplayParametersCommitKey )
+				continue;
+
+			if ( sym == gIODisplayParametersDefaultKey )
+				continue;
+
+			if ( !getSymbolRange( sym, &min, &max, &current ) )
+				continue;
+
+			IODisplay::setParameter( fDisplayParams, sym, current );
+			}
+
+		iter->release();
+		}
+
+	return( true );
+	}
+
+
+bool AppleVCP::doDataSet( const OSSymbol* paramName, OSData* value )
+	{
+	return( false );
+	}
+
+enum
+	{
+	kGetVCPFeature			= 0x01,
+	kReplyVCPFeature		= 0x02,
+	kSetVCPFeature			= 0x03,
+	kGetControlKeyMode		= 0x04,
+	kReplyControlKeyMode	= 0x05,
+	kSetControlKeyMode		= 0x06,
+	kGetTimingReport		= 0x07,
+	kGetKeyReport			= 0x08,
+	kResetVCPFeature		= 0x09,
+	kDisableVCPFeature		= 0x0A,
+	kEnableVCPFeature		= 0x0B,
+	kSaveCurrentSettings	= 0x0C
+	};
 
 // VCP minor commands from LGE
-enum {
-    kDegaussCommand		= 0x00,
-    kPresetDefaultCommand	= 0x04,
-    kExtBrightnessCommand	= 0x10,
-    kExtContrastCommand		= 0x12,
-    kRedDriveCommand		= 0x16,
-    kGreenDriveCommand		= 0x18,
-    kBlueDriveCommand		= 0x1A,
-    kHorizontalPositionCommand	= 0x20,
-    kHorizontalSizeCommand	= 0x22,
-    kPincushionCommand		= 0x24,
-    kPincushionBalanceCommand	= 0x26,
-    kVerticalPositionCommand	= 0x30,
-    kVerticalSizeCommand	= 0x32,
-    kParallelogramCommand	= 0x40,
-    kTrapezoidCommand		= 0x42,
-    kRotationCommand		= 0x44,
-    kRedCutoffCommand		= 0x80,
-    kGreenCutoffCommand		= 0x82,
-    kBlueCutoffCommand		= 0x84,
-    kSubContrastCommand		= 0x86,
-    kSubBrightnessCommand	= 0x88,
-    kABLCommand			= 0x8A,
-    kPresetStartCommand		= 0xE0,
-    kPresetEndCommand		= 0xE2,
-    kClearDataCommand		= 0xE4,
-    kSaveDataCommand		= 0xEA,
-    kSaveColorCommand		= 0xEB,
-    kSaveSystemDataCommand	= 0xEC
-};
+enum
+	{
+	kDegaussCommand				= 0x00,
+	kPresetDefaultCommand		= 0x04,
+	kExtBrightnessCommand		= 0x10,
+	kExtContrastCommand			= 0x12,
+	kRedDriveCommand			= 0x16,
+	kGreenDriveCommand			= 0x18,
+	kBlueDriveCommand			= 0x1A,
+	kHorizontalPositionCommand	= 0x20,
+	kHorizontalSizeCommand		= 0x22,
+	kPincushionCommand			= 0x24,
+	kPincushionBalanceCommand	= 0x26,
+	kVerticalPositionCommand	= 0x30,
+	kVerticalSizeCommand		= 0x32,
+	kParallelogramCommand		= 0x40,
+	kTrapezoidCommand			= 0x42,
+	kRotationCommand			= 0x44,
+	kRedCutoffCommand			= 0x80,
+	kGreenCutoffCommand			= 0x82,
+	kBlueCutoffCommand			= 0x84,
+	kSubContrastCommand			= 0x86,
+	kSubBrightnessCommand		= 0x88,
+	kABLCommand					= 0x8A,
+	kPresetStartCommand			= 0xE0,
+	kPresetEndCommand			= 0xE2,
+	kClearDataCommand			= 0xE4,
+	kSaveDataCommand			= 0xEA,
+	kSaveColorCommand			= 0xEB,
+	kSaveSystemDataCommand		= 0xEC
+	};
+
+enum
+	{
+	// this many byte to read to get vcp feature
+	kIICReadBufferSize			= 4,
+	kIICWriteBufferSize			= 7,
+	kIICSourceAddress			= 0x50,
+	kVCPCommandDataLength		= 0x84,
+
+	// special address that LGE check to not to go into master mode (Cuda can't do multi master mode)
+	kIICBogusSourceAddress		= 0xAF,
 
-enum {
-    // this many byte to read to get vcp feature
-    kIICReadBufferSize			= 4,
-    kIICWriteBufferSize			= 7,
-    kIICSourceAddress			= 0x50,
-    kVCPCommandDataLength		= 0x84,
-    // special address that LGE check to not to go into master mode (Cuda can't do multi master mode)
-    kIICBogusSourceAddress		= 0xAF,
     // Throttle to 30 times a second and round up a bit.
-    kMinMillisecondsBetweenIICCommands 	= 35
-};
+	kMinMillisecondsBetweenIICCommands 	= 35
+	};
 
-bool AppleVCP::readVCPRange( UInt8 command, SInt32 * oMax, SInt32 * current )
-{
-    UInt8	buffer[16];
-    IOByteCount count;
-    SInt32	max;
-    UInt32	retries;
 
-    retries = 0;
-    do {
-        buffer[0] = kIICBogusSourceAddress;
-        buffer[1] = kVCPCommandDataLength;
-        buffer[2] = kSetVCPFeature;
-        buffer[3] = command;
-        buffer[4] = 0;
-        buffer[5] = 0;
-        buffer[6] = fIICAddress
-                    ^ buffer[0]
-                    ^ buffer[1]
-                    ^ buffer[2]
-                    ^ buffer[3];
-    
-        count = kIICWriteBufferSize;
-        IOSleep( 100 ); 		// 100 in 9
-        AppleCudaWriteIIC( fIICAddress, buffer, &count );
-        bzero( buffer, sizeof( buffer));
-        count = kIICReadBufferSize;
-        IOSleep( 50 );		// 50 in 9
-        AppleCudaReadIIC( fIICAddress + 1, buffer, &count );
-        max = (buffer[0] << 8) | buffer[1];
+bool AppleVCP::readVCPRange( UInt8 command, SInt32* oMax, SInt32* oCurrent )
+	{
+	UInt8					buffer[ 16 ];
+	IOByteCount				count;
+	UInt32					retries;
+	SInt32					max;
+	SInt32					current;
 
-    } while( (max == 0) && (retries++ < 10));
+	retries = 0;
+	do
+    	{
+		buffer[ 0 ] = kIICBogusSourceAddress;
+		buffer[ 1 ] = kVCPCommandDataLength;
+		buffer[ 2 ] = kSetVCPFeature;
+		buffer[ 3 ] = command;
+		buffer[ 4 ] = 0;
+		buffer[ 5 ] = 0;
+		buffer[ 6 ] = fIICAddress ^ buffer[ 0 ] ^ buffer[ 1 ] ^ buffer[ 2 ] ^ buffer[ 3 ];
 
-    *oMax = max;
-    *current = (buffer[2] << 8) | buffer[3];
+		count = kIICWriteBufferSize;
+		IOSleep( 100 ); 		// 100 in 9
+		( void ) AppleCudaWriteIIC( fIICAddress, buffer, &count );
+
+		bzero( buffer, sizeof( buffer ) );
+		count = kIICReadBufferSize;
+		IOSleep( 50 );		// 50 in 9
+		( void ) AppleCudaReadIIC( fIICAddress + 1, buffer, &count );
+
+		max = ( buffer[ 0 ] << 8 ) | buffer[ 1 ];
+		current = ( buffer[ 2 ] << 8 ) | buffer[ 3 ];
+		}
+	while ( ( max == 0 ) && ( retries++ < 10 ) );
+
+	*oMax = max;
+	*oCurrent = current;
+
+	return( max != 0 );
+	}
 
-    return( max != 0 );
-}
 
 bool AppleVCP::setupVCP( void )
-{
-    OSIterator *	iter;
-    OSSymbol *		sym;
-    OSDictionary *	params;
-    OSNumber *		num;
-    UInt32		command;
-    SInt32	 	current, max;
+	{
+	OSIterator*						iter;
+	IOByteCount						zeroData;
+	IOReturn						err;
+	int								count;
 
-    IOByteCount		count;
-    IOReturn		err;
+	count = 0;
 
-    current = 0;
-    do {
-        count = 0;
-	IOSleep( 50 ); // 100 in 9
-        err = AppleCudaWriteIIC( fIICAddress, (UInt8 *) &count, &count );
-    } while( err && (current++ < 10));
-    if( kIOReturnSuccess != err) {
+	do
+		{
+        zeroData = 0;
+		IOSleep( 50 ); // 100 in 9
+		err = AppleCudaWriteIIC( fIICAddress, ( UInt8 * ) &zeroData, &zeroData );
+		}
+	while ( err && ( count++ < 10 ) );
+
+	if ( kIOReturnSuccess != err )
+		{
 #if DEBUG
-        IOLog("VCP warmup fail (%x)\n", err);
+		IOLog( "VCP warmup fail (%x)\n", err );
 #endif
-        return( false );
-    }
+		return( false );
+		}
 
-    iter = OSCollectionIterator::withCollection( fDisplayParams );
-    if( iter) {
-        while( (sym = (OSSymbol *) iter->getNextObject())) {
+	iter = OSCollectionIterator::withCollection( fDisplayParams );
+	if ( iter )
+		{
+		const OSSymbol*						sym;
 
-            if( sym == gIODisplayParametersCommitKey)
-                continue;
-            params = OSDynamicCast( OSDictionary, fDisplayParams->getObject(sym));
-            if( !params)
-                continue;
-            num = (OSNumber *) params->getObject("reg");
-            if( !num)
-                continue;
-            command = num->unsigned32BitValue();
-            if( !readVCPRange( command, &max, &current ))
-                continue;
-            IODisplay::addParameter( fDisplayParams, sym, 0, max );
-            IODisplay::setParameter( fDisplayParams, sym, current );
-        }
-        iter->release();
-    }
+		while( ( sym = ( const OSSymbol * ) iter->getNextObject() ) )
+			{
+			SInt32							current;
+			SInt32							max;
+			SInt32							min;
 
-    return( true );
-}
+			if ( sym == gIODisplayParametersCommitKey )
+				continue;
 
-bool AppleVCP::doIntegerSet( OSDictionary * params, const OSSymbol * sym, UInt32 value )
-{
-    OSNumber *	num;
-    UInt8	buffer[16];
-    IOByteCount count;
-    bool	ok = true;
-    UInt8	command;
+			if ( sym == gIODisplayParametersDefaultKey )
+				continue;
 
-    if( sym == gIODisplayParametersDefaultKey) {
-        return( true );
-    }
+			if ( !getSymbolRange( sym, &min, &max, &current ) )
+				continue;
 
-    if( !params)
-        return( true );
+			IODisplay::addParameter( fDisplayParams, sym, min, max );
+			IODisplay::setParameter( fDisplayParams, sym, current );
+			}
+		iter->release();
+		}
 
-    num = (OSNumber *) params->getObject("reg");
-    if( !num)
-        return( false );
-    command = num->unsigned32BitValue();
+	return( true );
+	}
 
-    if( sym == gIODisplayParametersCommitKey)
-        value = 0;
 
-    buffer[0] = kIICSourceAddress;
-    buffer[1] = kVCPCommandDataLength;
-    buffer[2] = kSetVCPFeature;
-    buffer[3] = command;
-    buffer[4] = 0;
-    buffer[5] = value;
-    buffer[6] = fIICAddress
-                ^ buffer[0]
-                ^ buffer[1]
-                ^ buffer[2]
-                ^ buffer[3]
-                ^ buffer[4]
-                ^ buffer[5];
+bool AppleVCP::doIntegerSet( OSDictionary* params, const OSSymbol* sym, UInt32 value )
+	{
+	UInt8								buffer[ 16 ];
+	OSNumber*							num;
+	IOByteCount							count;
+	IOReturn							err;
+	UInt8								command;
+	bool								ok = true;
 
-    if( sym == gIODisplayParametersCommitKey)
-        IOSleep( 110 );
-    else
-        IOSleep( 50 ); // 100 in 9
+	if ( !params )
+		{
+		return( true );
+		}
 
-    count = kIICWriteBufferSize;
-    AppleCudaWriteIIC( fIICAddress, buffer, &count );
+	if ( ( sym == gIODisplayParametersCommitKey ) || ( sym == gIODisplayParametersDefaultKey ) )
+		{
+		value = 0;
+		}
 
-    if( sym == gIODisplayParametersCommitKey)
-        IOSleep( 110 );
+	IOSleep( 50 ); // 100 in 9
+
+	if ( sym == gIODisplayParametersCommitKey )
+		{
+		// We need to issue two separate commands to save away the current graphics state: a kSaveDataCommand and
+		// kSaveSystemDataCommand.
+
+		command = kSaveDataCommand;
+		buffer[ 0 ] = kIICSourceAddress;
+		buffer[ 1 ] = kVCPCommandDataLength;
+		buffer[ 2 ] = kSetVCPFeature;
+		buffer[ 3 ] = command;
+		buffer[ 4 ] = 0;
+		buffer[ 5 ] = value;
+		buffer[ 6 ] = fIICAddress ^ buffer[ 0 ] ^ buffer[ 1 ] ^ buffer[ 2 ] ^ buffer[ 3 ] ^ buffer[ 4 ] ^ buffer[ 5 ];
+
+		count = kIICWriteBufferSize;
+
+		err = AppleCudaWriteIIC( fIICAddress, buffer, &count );
+
+		IOSleep( 50 ); // 100 in 9
+
+		command = kSaveSystemDataCommand;
+		buffer[ 0 ] = kIICSourceAddress;
+		buffer[ 1 ] = kVCPCommandDataLength;
+		buffer[ 2 ] = kSetVCPFeature;
+		buffer[ 3 ] = command;
+		buffer[ 4 ] = 0;
+		buffer[ 5 ] = value;
+		buffer[ 6 ] = fIICAddress ^ buffer[ 0 ] ^ buffer[ 1 ] ^ buffer[ 2 ] ^ buffer[ 3 ] ^ buffer[ 4 ] ^ buffer[ 5 ];
+
+		count = kIICWriteBufferSize;
+
+		err = AppleCudaWriteIIC( fIICAddress, buffer, &count );
+		}
+	else
+		{
+		num = ( OSNumber * ) params->getObject( "reg" );
+		if ( !num )
+			{
+			return( false );
+			}
+
+		command = num->unsigned32BitValue();
+
+		buffer[ 0 ] = kIICSourceAddress;
+		buffer[ 1 ] = kVCPCommandDataLength;
+		buffer[ 2 ] = kSetVCPFeature;
+		buffer[ 3 ] = command;
+		buffer[ 4 ] = 0;
+		buffer[ 5 ] = value;
+		buffer[ 6 ] = fIICAddress ^ buffer[ 0 ] ^ buffer[ 1 ] ^ buffer[ 2 ] ^ buffer[ 3 ] ^ buffer[ 4 ] ^ buffer[ 5 ];
+
+		count = kIICWriteBufferSize;
+
+		err = AppleCudaWriteIIC( fIICAddress, buffer, &count );
+		}
+
+	if ( ( sym == gIODisplayParametersCommitKey ) || ( sym == gIODisplayParametersDefaultKey ) )
+		IOSleep( 110 );
+	else
+		IOSleep( 50 );
+
+	return( ok );
+	}
 
-    return( ok );
-}
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
